/**
	$Id $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.extensions;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;

/**
	Wrapper for HttpServletRequest that allows us to set 
	proper servlet path and path info attributes for an extension
	servlet. The result should be the same as if the servlet had
	been mapped as a "standalone" servlet in web.xml.

	@author nicklas
	@version 2.10
	@base.modified $Date: 2009-06-15 11:05:06 +0200 (må, 15 jun 2009) $
*/
public class RequestWrapper
	extends HttpServletRequestWrapper
{

	private final String pathInfo;
	private final String servletPath;
	private final String originalServletPath;
	
	public RequestWrapper(HttpServletRequest request, String servletPath, String pathInfo)
	{
		super(request);
		this.pathInfo = pathInfo;
		this.servletPath = servletPath;
		this.originalServletPath = super.getServletPath();
	}

	/**
		From the HttpServletRequestInterface
		------------------------------------
	*/
	@Override
	public String getPathInfo()
	{
		return pathInfo;
	}
	@Override
	public String getServletPath()
	{
		// If the result of 'super.getServletPath()' has changed since
		// we created this wrapper, Tomcat is messing around with forward or
		// include requests
		String sp = super.getServletPath();
		return originalServletPath.equals(sp) ? servletPath : sp;
	}
	// ------------------------------------

	
}
