/**
	$Id: ColorFormatter.java 6319 2013-09-09 07:56:59Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.formatter;

import java.awt.Color;

import net.sf.basedb.util.ColorGenerator;
import net.sf.basedb.util.formatter.Formatter;

/**
	Formatter that adds a small colored box next to a value. The actual value
	is formatted by the supplied numeric formatter. The color to display is
	generated by a {@link ColorGenerator}. Null or Float NaN values are
	not colored.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2013-09-09 09:56:59 +0200 (må, 09 sep 2013) $
*/
public class ColorFormatter
	implements Formatter<Number>
{

	private ColorGenerator colorGen;
	private Formatter<Number> numberFormatter;
	
	/**
		Create a new color formatter.

		@param colorGen The color generator to use for generating colors
		@param numberFormatter The formatter to use for formatterin the values
	*/
	public ColorFormatter(ColorGenerator colorGen, Formatter<Number> numberFormatter)
	{
		this.colorGen = colorGen;
		this.numberFormatter = numberFormatter;
	}
	
	/*
		From the Formatter interface
		-------------------------------------------
	*/
	@Override
	public String format(Number value)
	{
		StringBuilder sb = new StringBuilder();
		sb.append("<div ");
		Color color = value != null && !Float.isNaN(value.floatValue()) ? colorGen.getColor(value.floatValue()) : null;
		if (color != null)
		{
			sb.append("class=\"colorbox inline\" style=\"background: #");
			sb.append(ColorGenerator.toHex(color));
			sb.append(";\"");
		}
		else
		{
			sb.append("class=\"colorbox inline nan\"");
		}
		sb.append("></div> ");
		sb.append(numberFormatter.format(value));
		return sb.toString();
	}

	@Override
	public Number parseString(String value)
	{
		return numberFormatter.parseString(value);
	}
	// -------------------------------------------

}
