/*
	$Id: AscOrder.java 6127 2012-09-14 07:28:35Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.query;

import java.util.Collection;
import java.util.Collections;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.util.EqualsHelper;

/**
	Sort the result of a query in ascending order by the expression.

	@author Nicklas
	@version 2.0
	@see Orders#asc(Expression)
	@base.modified $Date: 2012-09-14 09:28:35 +0200 (fr, 14 sep 2012) $
*/
public class AscOrder
	implements Order
{
	private final Expression expression;

	AscOrder(Expression expression)
	{
		assert expression != null : "expression == null";
		this.expression = expression;
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	@Override
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		return expression.toQl(query, dc)+" ASC";
	}
	/**
		@return The expression to order by
	*/
	@Override
	public Collection<? extends QueryElement> getChildren()
	{
		return Collections.singleton(expression);
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	@Override
	public String toString()
	{
		return expression + " ASC";
	}
	/**
		This expression is equal to another AscOrder if their
		child expressions are equal.
	*/
	@Override
	public boolean equals(Object other)
	{
		if (this == other) return true;
		if (other == null || this.getClass() != other.getClass()) return false;
		AscOrder o = (AscOrder)other;
		return EqualsHelper.equals(expression, o.expression);
	}
	@Override
	public int hashCode()
	{
		return EqualsHelper.hashCode(expression) + getClass().hashCode();
	}
	// -------------------------------------------
}

