/**
	$Id: FileImportInputStream.java 5353 2010-05-27 08:54:05Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util;

import net.sf.basedb.core.File;
import net.sf.basedb.core.plugin.ImportInputStream;

/**
 	An extension to the {@link ImportInputStream} class which 
 	reads from a file on the BASE file system.

	@author nicklas
	@version 2.9
	@base.modified $Date: 2010-05-27 10:54:05 +0200 (to, 27 maj 2010) $
*/
public class FileImportInputStream
	extends ImportInputStream
{
	private final File file;
	private final String charset;

	/**
		Create a new import stream that reads data from 
		a file in the BASE filesystem.
		@param file The file to read from
		@param offset The offset (in bytes) to start reading from
		@param charset The character set to use when reading the file,
			or null to use {@link File#getCharacterSet()}.
	*/
	public FileImportInputStream(File file, long offset, String charset)
	{
		super(offset == 0 ? file.getCachedDownloadStream() : file.getDownloadStream(offset));
		this.file = file;
		this.charset = charset == null ? file.getCharacterSet() : charset;
	}
	
	/*
	 	From the ImportInputStream class
	 	---------------------------------
	*/
	/**
		Calls {@link File#getSize()} to get the file size.
	*/
	@Override
	public long getLength()
	{
		return file.getSize();
	}
	/**
		Calls {@link File#getMimeType()}.
	*/
	@Override
	public String getMimeType()
	{
		return file.getMimeType();
	}
	/**
		Calls {@link File#getCharacterSet()}, unless a character set
		was specified in the constructor.
	*/
	@Override
	public String getCharacterSet()
	{
		return charset;
	}
	/**
		Calls {@link File#getName()}.
	*/
	@Override
	public String getFilename()
	{
		return file.getName();
	}
	// -------------------------------------------
}
