/**
	$Id: DataWriterFactory.java 5198 2009-12-15 12:56:17Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.bfs;

import java.io.IOException;

/**
	Factory implementations are responsible for creating BFS data writers
	that accepts data for a given owner object. 

	@author Nicklas
	@version 2.15
	@base.modified $Date: 2009-12-15 13:56:17 +0100 (ti, 15 dec 2009) $
*/
public interface DataWriterFactory<T>
{
	/**
		Create a data writer for writing data that belongs to the
		given owner. The owner is typically a bioassay (serial bfs format)
		or a spot data field (matrix bfs format). The returned writer should
		usually have it's filename ({@link DataWriter#getFilename()}) and
		number of columns ({@link DataWriter#getColumnCount()}) set, since this 
		is needed for the metadata generation.
		
		@param owner The object that "owns" the data that is written
			to the file
		@param suggestedFilename An optional suggested filename, the
			factory may use this string or generate a different filename
		@return A data writer object with the filename and number 
			of colums set (unless otherwise specified)
	*/
	public DataWriter createDataWriter(T owner, String suggestedFilename)
		throws IOException;
}
