/*
	$Id $
	
	Copyright (C) 2010 Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	
	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	at your option) any later version.
	
	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.BioAssay;
import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Directory;
import net.sf.basedb.core.File;
import net.sf.basedb.core.Formula;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.Transformation;
import net.sf.basedb.core.Type;
import net.sf.basedb.core.VirtualColumn;
import net.sf.basedb.core.query.Dynamic;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.util.ConsoleProgressReporter;
import net.sf.basedb.util.FileUtil;
import net.sf.basedb.util.Values;
import net.sf.basedb.util.bfs.AnnotationModel;
import net.sf.basedb.util.bfs.AnnotationParser;
import net.sf.basedb.util.bfs.AnnotationWriter;
import net.sf.basedb.util.bfs.BaseInputStreamLocator;
import net.sf.basedb.util.bfs.BfsParser;
import net.sf.basedb.util.bfs.DataParser;
import net.sf.basedb.util.bfs.DataWriter;
import net.sf.basedb.util.bfs.EventHandler;
import net.sf.basedb.util.bfs.EventType;
import net.sf.basedb.util.bfs.GenericExtraFileImporter;
import net.sf.basedb.util.bfs.GenericBaseDataWriterFactory;
import net.sf.basedb.util.bfs.MetadataModel;
import net.sf.basedb.util.bfs.MetadataParser;
import net.sf.basedb.util.bfs.MetadataWriter;
import net.sf.basedb.util.bfs.SuggestedFilenameGenerator;
import net.sf.basedb.util.bfs.MetadataModel.Section;
import net.sf.basedb.util.export.spotdata.AssayField;
import net.sf.basedb.util.export.spotdata.BfsExporter;
import net.sf.basedb.util.export.spotdata.DynamicField;
import net.sf.basedb.util.export.spotdata.ExportableFieldConverter;
import net.sf.basedb.util.export.spotdata.ExportableFieldFactory;
import net.sf.basedb.util.export.spotdata.MatrixBfsExporter;
import net.sf.basedb.util.export.spotdata.SerialBfsExporter;
import net.sf.basedb.util.export.spotdata.StandardFieldConverter;
import net.sf.basedb.util.importer.spotdata.BfsImporter;

import java.util.ArrayList;
import java.util.List;


public class TestBfsExporterImporter
{
	static boolean ok = true;
	
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		boolean wasSetup = TestAnalyzeUtil.setupExperiment();
		write("++Testing BFS exporter and importer");
		try
		{
			int experimentId = TestAnalyzeUtil.getItemId("experiment.genepix");
			int rootBasId = TestAnalyzeUtil.getItemId("bioassayset.root");
			int tempFactorId = TestAnalyzeUtil.getItemId("factor.temperature");
			
			if (experimentId == 0 || rootBasId == 0)
			{
				ok = false;
				write("Test experiment was not setup correctly.");
				return ok;
			}
			
			int assayAnnotationId = TestAnnotationType.test_create("Assay mean", Type.FLOAT, 0, null, null, Item.BIOASSAY, 1, null, false);
			
			int bfsSerialDir = TestDirectory.test_create(true, "bfs-serial");
			int bfsMatrixDir = TestDirectory.test_create(true, "bfs-matrix");
			int bfsMergeAssaysDir = TestDirectory.test_create(true, "bfs-merge-assays");
			int bfsMergeReportersDir = TestDirectory.test_create(true, "bfs-merge-reporters");
	
			int exportRoot = TestAnalyzeUtil.clone_bioassay_set(rootBasId, "BfsExporter");
			test_annotate_assays(exportRoot, assayAnnotationId, 4.5f, 7.8f);
			
			test_export(exportRoot, bfsSerialDir, "Temperature", "Assay mean", false);
			test_export(exportRoot, bfsMatrixDir, "Temperature", "Assay mean", true);
			
			test_import(exportRoot, bfsSerialDir, false, "Imported Serial BFS");
			test_import(exportRoot, bfsMatrixDir, true, "Imported Matrix BFS");
	
			test_upload_extra_file("data/test.upload.txt", "x-table.txt", bfsMergeAssaysDir);
			test_upload_extra_file("data/test.upload.png", "x-plot.png", bfsMergeAssaysDir);
			
			test_merge_assays(bfsMatrixDir, bfsMergeAssaysDir);
			test_import(exportRoot, bfsMergeAssaysDir, true, "Merged assays");
			test_merge_reporters(bfsMatrixDir, bfsMergeReportersDir);
			test_import(exportRoot, bfsMergeReportersDir, true, "Merged reporters");

			if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
			TestAnalyzeUtil.test_delete_bioassayset(exportRoot);
			TestDirectory.test_delete(bfsSerialDir, true);
			TestDirectory.test_delete(bfsMatrixDir, true);
			TestDirectory.test_delete(bfsMergeAssaysDir, true);
			TestDirectory.test_delete(bfsMergeReportersDir, true);
			TestAnnotationType.test_delete(assayAnnotationId);
		}
		finally
		{
			if (wasSetup) TestAnalyzeUtil.cleanupExperiment();
			write("++Testing BFS exporter and importer "+(ok ? "OK" : "Failed")+"\n");
		}
		return ok;
	}
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	static void test_export(int bioAssaySetId, int directoryId, 
			String experimentalFactor, String assayAnnotation, boolean matrix)
	{
		if (bioAssaySetId == 0 || directoryId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioAssaySet bas = BioAssaySet.getById(dc, bioAssaySetId);
			Directory dir = Directory.getById(dc, directoryId);
			
			// Create exporter
			BfsExporter bfs = matrix ? new MatrixBfsExporter() : new SerialBfsExporter();
			bfs.setDbControl(dc);
			bfs.setSource(bas);
			if (!TestUtil.getSilent()) bfs.setProgressReporter(new ConsoleProgressReporter());

			// Create files and file factory
			File metadata = createFile(dc, dir, "metadata.txt");
			File reporter = createFile(dc, dir, "rdata.txt");
			File assay = createFile(dc, dir, "pdata.txt");
			bfs.setMetadataWriter(MetadataWriter.create(
					metadata.getUploadStream(false), metadata.getName()));
			bfs.setReporterAnnotationsWriter(AnnotationWriter.create(
					reporter.getUploadStream(false), reporter.getName()));
			bfs.setAssayAnnotationsWriter(AnnotationWriter.create(
					assay.getUploadStream(false), assay.getName()));
			bfs.setDataWriterFactory(new GenericBaseDataWriterFactory(dc, dir, new SuggestedFilenameGenerator()));
			
			// Parameters that go in the 'settings' section
			bfs.setParameter("b", "some parameter");
			bfs.setParameter("a", "some other parameter");
			
			ExportableFieldConverter fieldConverter = new StandardFieldConverter(dc, bas);
			
			// Configure the spot fields to export
			
			List<DynamicField> spotFields = new ArrayList<DynamicField>();
			spotFields.add(fieldConverter.getSpotField("Ch 1", true));
			spotFields.add(fieldConverter.getSpotField("Ch 2", true));
			spotFields.add(fieldConverter.getSpotField("row", true));
			spotFields.add(fieldConverter.getSpotField("flags", true));
			spotFields.add(fieldConverter.getSpotField("Ratio, ch1 / ch2", true));
			bfs.addSpotFields(spotFields);
			
			// Configure the reporter fields to export
			List<DynamicField> reporterFields = new ArrayList<DynamicField>();
			reporterFields.add(ExportableFieldFactory.simple(Dynamic.column(VirtualColumn.POSITION), "Position", Type.INT, Formula.AverageMethod.NONE, null));
			reporterFields.add(fieldConverter.getReporterField("Name", true));
			reporterFields.add(fieldConverter.getReporterField("External ID", true));
			bfs.addReporterFields(reporterFields);

			// Configure assay fields to export
			List<AssayField> assayFields = new ArrayList<AssayField>();
			assayFields.add(fieldConverter.getAssayField("Name", true));
			if (experimentalFactor != null)
			{
				assayFields.add(fieldConverter.getAssayField(experimentalFactor, true));
			}
			if (assayAnnotation != null)
			{
				assayFields.add(fieldConverter.getAssayField(assayAnnotation, true));
			}
			bfs.addAssayFields(assayFields);
			
			bfs.doExport();
			if (!TestUtil.getSilent()) write("");
			dc.commit();
			write("--Exporting bfs [" + (matrix ? "matrix" : "serial") + "] OK");
		}
		catch (Throwable ex)
		{
			write("--Exporting bfs [" + (matrix ? "matrix" : "serial") + "] FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}

	}
	

	static int test_import(int bioAssaySetId, int directoryId, boolean matrix, String childName)
	{
		DbControl dc = null;
		if (bioAssaySetId == 0 || directoryId == 0) return 0;
		int id = 0;
		try
		{
			dc = TestUtil.getDbControl();
			BioAssaySet bas = BioAssaySet.getById(dc, bioAssaySetId);
			Directory dir = Directory.getById(dc, directoryId);
			File metadata = File.getFile(dc, dir, "metadata.txt", false);
			Transformation t = bas.newTransformation(null);
			t.setName("Importing " + (matrix ? "matrix" : "serial") + " BFS");
			
			BfsImporter bfs = new BfsImporter();
			bfs.setDbControl(dc);
			bfs.setTransformation(t);
			dc.saveItem(t);
			bfs.setMetadataParser(MetadataParser.create(metadata));
			bfs.setInputStreamLocator(new BaseInputStreamLocator(dc, dir));
			bfs.setExtraFileImporter(new GenericExtraFileImporter(dc, dir, false));
			if (!TestUtil.getSilent()) bfs.setProgressReporter(new ConsoleProgressReporter());
			BioAssaySet child = bfs.doImport();
			if (!TestUtil.getSilent()) write("\n");
			child.setName(childName);
			dc.commit();
			
			id = child.getId();
			write("--Importing bfs [" + (matrix ? "matrix" : "serial") + "] OK");
		}
		catch (Throwable ex)
		{
			write("--Importing bfs [" + (matrix ? "matrix" : "serial") + "] FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static int test_merge_assays(int sourceDirId, int mergeDirId)
	{
		DbControl dc = null;
		if (sourceDirId == 0 || mergeDirId == 0) return 0;
		int id = 0;
		try
		{
			dc = TestUtil.getDbControl();
			Directory sourceDir = Directory.getById(dc, sourceDirId);
			Directory mergeDir = Directory.getById(dc, mergeDirId);
			
			File ch1In = File.getFile(dc, sourceDir, "Ch 1.txt", false);
			File ch2In = File.getFile(dc, sourceDir, "Ch 2.txt", false);
			File reporterIn = File.getFile(dc, sourceDir, "rdata.txt", false);
			File assaysIn = File.getFile(dc, sourceDir, "pdata.txt", false);
			
			// Create the new metadata file
			File metadataOut = File.getFile(dc, mergeDir, "metadata.txt", true);
			dc.saveItem(metadataOut);
			MetadataModel metadata = new MetadataModel();
			metadata.setSubtype("matrix");
			metadata.addFile("sdata1", "Ch 1.txt");
			metadata.addFile("sdata2", "Ch 2.txt");
			metadata.addFile("pdata", "pdata.txt");
			metadata.addFile("rdata", "rdata.txt");

			// Extra files
			ItemQuery<File> query = mergeDir.getFiles();
			query.restrict(Restrictions.like(Hql.property("name"), Expressions.string("x-%")));
			List<File> extraFiles = query.list(dc);
			for (File f : extraFiles)
			{
				metadata.addFile(f.getName(), f.getName());
			}
			
			Section sdata = metadata.addSection("sdata");
			sdata.addEntry("Ch 1", "float");
			sdata.addEntry("Ch 2", "float");
			Section settings = metadata.addSection("settings");
			settings.addEntry("new-data-cube", "1");
			settings.addEntry("multi-assay-parents", "1");
			MetadataWriter metadataWriter = MetadataWriter.create(metadataOut);
			metadata.print(metadataWriter);
			metadataWriter.close();
			
			// The reporter annotations file is simply copied since we keep the positions
			File reporterOut = File.getFile(dc, mergeDir, "rdata.txt", true);
			dc.saveItem(reporterOut);
			FileUtil.copy(reporterIn.getDownloadStream(0), reporterOut.getUploadStream(false));
			
			// We create a new assay annotations file
			File assaysOut = File.getFile(dc, mergeDir, "pdata.txt", true);
			dc.saveItem(assaysOut);
			AnnotationModel assayAnnotations = new AnnotationModel();
			AnnotationParser assayParser = AnnotationParser.create(assaysIn);
			assayParser.parse(assayAnnotations);
			AnnotationWriter assayWriter = AnnotationWriter.create(assaysOut);
			assayWriter.bfsPrintHeaders("Parent ID", "Name");
			StringBuilder sb = new StringBuilder();
			for (int i = 0; i < assayAnnotations.getRowCount(); ++i)
			{
				if (i > 0) sb.append(",");
				sb.append(assayAnnotations.getRowId(i));
			}
			assayWriter.bfsPrintData(1, sb.toString(), "Merged from " + sb.toString());
			assayWriter.close();
			
			File ch1Out = File.getFile(dc, mergeDir, "Ch 1.txt", true);
			dc.saveItem(ch1Out);
			DataWriter ch1Writer = DataWriter.create(ch1Out);
			DataParser ch1Parser = DataParser.create(ch1In);
			ch1Parser.parse(new MergeAssaysEventHandler(ch1Writer));
			ch1Writer.close();

			File ch2Out = File.getFile(dc, mergeDir, "Ch 2.txt", true);
			dc.saveItem(ch2Out);
			DataWriter ch2Writer = DataWriter.create(ch2Out);
			DataParser ch2Parser = DataParser.create(ch2In);
			ch2Parser.parse(new MergeAssaysEventHandler(ch2Writer));
			ch2Writer.close();

			dc.commit();
			
			write("--Merge assays OK");
		}
		catch (Throwable ex)
		{
			write("--Merge assays FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static int test_merge_reporters(int sourceDirId, int mergeDirId)
	{
		DbControl dc = null;
		if (sourceDirId == 0 || mergeDirId == 0) return 0;
		int id = 0;
		try
		{
			dc = TestUtil.getDbControl();
			Directory sourceDir = Directory.getById(dc, sourceDirId);
			Directory mergeDir = Directory.getById(dc, mergeDirId);
			
			File ch1In = File.getFile(dc, sourceDir, "Ch 1.txt", false);
			File ch2In = File.getFile(dc, sourceDir, "Ch 2.txt", false);
			File reporterIn = File.getFile(dc, sourceDir, "rdata.txt", false);
			File assaysIn = File.getFile(dc, sourceDir, "pdata.txt", false);
			
			// Create the new metadata file
			File metadataOut = File.getFile(dc, mergeDir, "metadata.txt", true);
			dc.saveItem(metadataOut);
			MetadataModel metadata = new MetadataModel();
			metadata.setSubtype("matrix");
			metadata.addFile("sdata1", "Ch 1.txt");
			metadata.addFile("sdata2", "Ch 2.txt");
			metadata.addFile("pdata", "pdata.txt");
			metadata.addFile("rdata", "rdata.txt");
			Section sdata = metadata.addSection("sdata");
			sdata.addEntry("Ch 1", "float");
			sdata.addEntry("Ch 2", "float");
			Section settings = metadata.addSection("settings");
			settings.addEntry("new-data-cube", "1");
			MetadataWriter metadataWriter = MetadataWriter.create(metadataOut);
			metadata.print(metadataWriter);
			metadataWriter.close();
			
			// The assay annotations file is simply copied since we keep the parents
			File assaysOut = File.getFile(dc, mergeDir, "pdata.txt", true);
			dc.saveItem(assaysOut);
			FileUtil.copy(assaysIn.getDownloadStream(0), assaysOut.getUploadStream(false));
			
			// We create a new reporter annotations file
			File reportersOut = File.getFile(dc, mergeDir, "rdata.txt", true);
			dc.saveItem(reportersOut);
			AnnotationModel reporterAnnotations = new AnnotationModel();
			AnnotationParser reporterParser = AnnotationParser.create(reporterIn);
			reporterParser.parse(reporterAnnotations);
			AnnotationWriter reporterWriter = AnnotationWriter.create(reportersOut);
			reporterWriter.bfsPrintHeaders("External ID");
			int externalIdIndex = reporterAnnotations.getColumnIndex("External ID");
			for (int i = 0; i < reporterAnnotations.getRowCount(); i+=2)
			{
				reporterWriter.bfsPrintData(1 + i / 2, reporterAnnotations.getData(i, externalIdIndex));
			}
			reporterWriter.close();
			
			File ch1Out = File.getFile(dc, mergeDir, "Ch 1.txt", true);
			dc.saveItem(ch1Out);
			DataWriter ch1Writer = DataWriter.create(ch1Out);
			DataParser ch1Parser = DataParser.create(ch1In);
			ch1Parser.parse(new MergeReportersEventHandler(ch1Writer));
			ch1Writer.close();

			File ch2Out = File.getFile(dc, mergeDir, "Ch 2.txt", true);
			dc.saveItem(ch2Out);
			DataWriter ch2Writer = DataWriter.create(ch2Out);
			DataParser ch2Parser = DataParser.create(ch2In);
			ch2Parser.parse(new MergeReportersEventHandler(ch2Writer));
			ch2Writer.close();

			dc.commit();
			
			write("--Merge assays OK");
		}
		catch (Throwable ex)
		{
			write("--Merge assays FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}
	
	static File createFile(DbControl dc, Directory dir, String name)
	{
		File f = File.getNew(dc, dir);
		f.setName(name);
		f.setCharacterSet("UTF-8");
		f.setMimeType("text/plain");
		dc.saveItem(f);
		return f;
	}
	
	static int test_upload_extra_file(String file, String name, int directoryId)
	{
		if (directoryId == 0) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Directory d = Directory.getById(dc, directoryId);
			File f = File.getNew(dc, d);
			f.setName(name);
			f.setMimeTypeAuto(null, null);
			dc.saveItem(f);			
			FileUtil.copy(FileUtil.getInputStream(new java.io.File(file)), f.getUploadStream(false));
			dc.commit();
			write("--Upload extra file '" + file + "' OK");
		}
		catch (Throwable ex)
		{
			write("--Upload extra file '" + file + "' FAILED");
			ex.printStackTrace();
			ok = false;
		}

		finally
		{
			if (dc != null) dc.close();
		}
		return id;
		
	}
	
	static void test_annotate_assays(int bioassaySetId, int annotationTypeId, Object... values)
	{
		if (bioassaySetId == 0 || annotationTypeId == 0) return;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioAssaySet bas = BioAssaySet.getById(dc, bioassaySetId);
			AnnotationType at = AnnotationType.getById(dc, annotationTypeId);
			
			int i = 0;
			int count = 0;
			for (BioAssay ba : bas.getBioAssays().list(dc))
			{
				Object value = values[i];
				ba.getAnnotationSet().getAnnotation(at).setValue(value);
				++i;
				if (i >= values.length) i = 0;
				count++;
			}
			
			dc.commit();
			write("--Annotated " + count + " assays OK");
		}
		catch (Throwable ex)
		{
			write("--Annotated assays FAILED");
			ex.printStackTrace();
			ok = false;
		}

		finally
		{
			if (dc != null) dc.close();
		}
		return;
	}
	
	static class MergeAssaysEventHandler
		implements EventHandler
	{
		
		private DataWriter dataWriter;
		
		MergeAssaysEventHandler(DataWriter dataWriter)
		{
			this.dataWriter = dataWriter;
		}

		@SuppressWarnings("rawtypes")
		@Override
		public void handleEvent(EventType eventType, Object eventData,
				BfsParser parser)
		{

			if (eventType == DataParser.DATA_EVENT)
			{
				String[] data = (String[])eventData;
				double sum = 0.0;
				for (int i = 0; i < data.length; ++i)
				{
					sum += Values.getDouble(data[0], Double.NaN);
				}
				dataWriter.bfsPrintData(sum / data.length);
			}
			
		}
	}
		
	static class MergeReportersEventHandler
		implements EventHandler
	{
		
		private DataWriter dataWriter;
		private String[] line1;
		
		MergeReportersEventHandler(DataWriter dataWriter)
		{
			this.dataWriter = dataWriter;
		}

		@SuppressWarnings("rawtypes")
		@Override
		public void handleEvent(EventType eventType, Object eventData,
				BfsParser parser)
		{

			if (eventType == DataParser.DATA_EVENT)
			{
				String[] data = (String[])eventData;
				if (line1 == null)
				{
					line1 = data;
				}
				else
				{
					Object[] newData = new Object[data.length];
					for (int i = 0; i < data.length; ++i)
					{
						float sum = Values.getFloat(line1[i], Float.NaN) + Values.getFloat(data[i], Float.NaN);
						newData[i] = sum / 2;
					}
					dataWriter.bfsPrintData(newData);
				}
			}
			
		}
	
	}
}
