<%-- $Id: edit_group.jsp 6412 2014-02-04 12:07:38Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------


	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.SystemItems"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.Type"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Include"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.Group"
	import="net.sf.basedb.core.User"
	import="net.sf.basedb.core.Quota"
	import="net.sf.basedb.core.QuotaType"
	import="net.sf.basedb.core.Location"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.core.query.Restrictions"
	import="net.sf.basedb.core.query.Expressions"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="net.sf.basedb.clients.web.extensions.edit.EditUtil"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
	import="net.sf.basedb.util.json.JsonUtil"
	import="net.sf.basedb.util.json.NameableConverter"
	import="java.util.Arrays"
	import="org.json.simple.JSONArray"
	import="org.json.simple.JSONObject"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%@ taglib prefix="ext" uri="/WEB-INF/extensions.tld" %>
<%
final Item itemType = Item.GROUP;
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);
final int itemId = cc.getId();
final String ID = sc.getId();
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl();
try
{
	String title = null;
	Group group = null;

	boolean isEveryone = false;
	boolean isDefault = false;
	boolean hiddenMembers = false;
	final QuotaType total = QuotaType.getById(dc, SystemItems.getId(QuotaType.TOTAL));
	Quota currentQuota = null;
	
	// Query to retrieve child groups
	ItemQuery<Group> groupQuery = null;
	// Query to retrieve user members
	ItemQuery<User> userQuery = null;

	if (itemId == 0)
	{
		title = "Create group";
		cc.removeObject("item");
		if (cc.getPropertyFilter("quota.name") != null)
		{
			currentQuota = Base.getFirstMatching(dc, Quota.getQuery(), "name", cc.getPropertyFilter("quota.name"));
		}
		String[] selected = request.getParameterValues("user_id");
		if (selected != null)
		{
			userQuery = User.getQuery();
			userQuery.include(Include.ALL);
			userQuery.restrict(Restrictions.in(Hql.property("id"), Expressions.parameter("selected")));
			userQuery.setParameter("selected", Arrays.asList(Values.getInt(selected)), Type.INT);
		}
		isDefault = Values.getBoolean(cc.getPropertyValue("default"), false);
		hiddenMembers = Values.getBoolean(cc.getPropertyValue("hiddenMembers"), false);
	}
	else
	{
		group = Group.getById(dc, itemId);
		group.checkPermission(Permission.WRITE);
		cc.setObject("item", group);
		title = "Edit group -- " + HTML.encodeTags(group.getName());
		isEveryone = Group.EVERYONE.equals(group.getSystemId());
		isDefault = group.isDefault();
		hiddenMembers = group.hasHiddenMembers();

		try
		{
			currentQuota = group.getQuota();
		}
		catch (PermissionDeniedException ex)
		{}
		
		groupQuery = group.getGroups();
		groupQuery.include(Include.ALL);
		groupQuery.order(Orders.asc(Hql.property("name")));
		userQuery = group.getUsers();
		userQuery.include(Include.ALL);
		userQuery.order(Orders.asc(Hql.property("name")));
	}
	
	final boolean readQuota = sc.hasPermission(Permission.READ, Item.QUOTA);
	final boolean useQuota = sc.hasPermission(Permission.USE, Item.QUOTA);
	final boolean useUsers = sc.hasPermission(Permission.WRITE, Item.USER);
	final boolean writeGroups = sc.hasPermission(Permission.WRITE, Item.GROUP);
	final boolean writeMembers = useUsers && writeGroups;
	
	// Query to retrieve quota
	final ItemQuery<Quota> quotaQuery = Quota.getQuery();
	quotaQuery.include(Include.ALL);
	quotaQuery.order(Orders.asc(Hql.property("name")));
	quotaQuery.setCacheResult(true);
	
	// Load user members as JSON objects
	JSONObject jsonUsers = new JSONObject();
	jsonUsers.put("itemType", "USER");
	jsonUsers.put("name", "Users");
	if (userQuery != null)
	{
		jsonUsers.put("items", JsonUtil.toArray(userQuery.iterate(dc), new NameableConverter()));
	}

	// Load group members as JSON objects
	JSONObject jsonGroups = new JSONObject();
	jsonGroups.put("itemType", "GROUP");
	jsonGroups.put("name", "Groups");
	if (groupQuery != null)
	{
		jsonGroups.put("items", JsonUtil.toArray(groupQuery.iterate(dc), new NameableConverter()));
	}
	
	JSONArray jsonMembers = new JSONArray();
	jsonMembers.add(jsonUsers);
	jsonMembers.add(jsonGroups);

	JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, GuiContext.item(itemType), group);
	ExtensionsInvoker invoker = EditUtil.useEditExtensions(jspContext);
	%>
	<base:page type="popup" title="<%=title%>" id="edit-page">
	<base:head scripts="tabcontrol-2.js,linkitems-2.js,~groups.js" styles="tabcontrol.css">
		<ext:scripts context="<%=jspContext%>" />
		<ext:stylesheets context="<%=jspContext%>" />
	</base:head>
	<base:body>
		<h1><%=title%> <base:help tabcontrol="settings" /></h1>

		<form action="index.jsp?ID=<%=ID%>" method="post" name="group">
		<input type="hidden" name="cmd" value="UpdateItem">

		<t:tabcontrol id="settings" 
			subclass="content dialogtabcontrol"
			position="bottom" remember="<%=group != null%>"
			extensions="<%=invoker%>">
		<t:tab id="info" title="Group" helpid="group.edit">
			<table class="fullform input100">
			<tr>
				<th>Name</th>
				<td><input class="text required auto-init" data-auto-init="<%=group == null ? "focus-select" : "focus" %>"
					type="text" name="name" 
					value="<%=HTML.encodeTags(group == null ? Values.getString(cc.getPropertyValue("name"), "New group") : group.getName())%>" 
					maxlength="<%=Group.MAX_NAME_LENGTH%>"></td>
				<td></td>
			</tr>
			<tr>
				<th>Default</th>
				<td>
					<input type="radio" name="is_default" id="defaultNo" value="0" <%=!isDefault ? "checked" : ""%>><label for="defaultNo">no</label>
					<input type="radio" name="is_default" id="defaultYes" value="1" <%=isDefault ? "checked" : ""%>><label for="defaultYes">yes</label>
				</td>
				<td></td>
			</tr>	
			<tr>
				<th>Hidden members</th>
				<td>
					<input type="radio" name="hidden_members" id="hiddenNo" value="0" <%=!hiddenMembers ? "checked" : ""%>><label for="hiddenNo">no</label>
					<input type="radio" name="hidden_members" id="hiddenYes" value="1" <%=hiddenMembers ? "checked" : ""%>><label for="hiddenYes">yes</label>
				</td>
				<td></td>
			</tr>	
			<tr>
				<th>Quota</th>
				<td>
					<select name="quota_id" <%=isEveryone || !useQuota ? "disabled readonly class=\"disabled\"" : ""%>>
					<%
					if (!readQuota)
					{
						%>
						<option value="-1">- denied -
						<%
					}
					else
					{
						%>
						<option value="0">- none -
						<%
						ItemResultList<Quota> quotas = quotaQuery.list(dc);
						for (Quota quota : quotas)
						{
							boolean current = quota.equals(currentQuota);
							if (!current && quota.isRemoved()) continue;
							int id = quota.getId();
							long totalBytes = quota.getQuotaValue(total, Location.PRIMARY);
							String fTotal = totalBytes == Quota.UNLIMITED ? "unlimited" : Values.formatBytes(totalBytes);
							%>
							<option 
								value="<%=current && group != null ? -id : id%>" 
								<%=current ? "selected" : ""%>
								><%=HTML.encodeTags(quota.getName())%> (<%=fTotal%> total)
							<%
						}
					}
					%>
					</select>
				</td>
				<td></td>
			</tr>
			<tr class="dynamic">
				<th>Description</th>
				<td>
					<textarea class="text" rows="6" name="description" id="description"
						><%=HTML.encodeTags(group == null ? cc.getPropertyValue("description") : group.getDescription())%></textarea>
				</td>
				<td style="width: 20px;">
					<base:zoom textarea="description" title="Description" />
				</td>
			</tr>
			</table>
		</t:tab>
		
		<t:tab id="members" title="Members" tooltip="Add/remove members of this group" 
			helpid="group.edit.membership">
			<table class="fullform input100">
			<tr class="dynamic">
			<th>Members</th>
			<td>
				<div class="selectionlist">
					<table>
					<tr>
						<td>
							<select name="members" id="members" 
								class="auto-init" data-auto-init="link-container"
								data-initial-items="<%=HTML.encodeTags(jsonMembers.toJSONString()) %>"
								data-initial-action="<%=group == null ? 1 : 0%>"
								size="14" multiple <%=isEveryone || !writeMembers ? "disabled readonly class=\"disabled\"" : ""%>>
							</select>
						</td>
						<td style="vertical-align: top;">
							
							<base:buttongroup vertical="true">
								<base:button 
									id="btnAddUsers"
									subclass="leftaligned auto-init"
									data-auto-init="add-link"
									data-list-id="members"
									data-item-type="USER"
									style="width: 12em;"
									title="Add&nbsp;users&hellip;" 
									tooltip="Add users to this group"
									disabled="<%=isEveryone || !writeMembers %>" 
								/>
								<base:button 
									id="btnAddGroups"
									subclass="leftaligned auto-init"
									data-auto-init="add-link"
									data-list-id="members"
									data-item-type="GROUP"
									data-exclude="<%=group==null ? 0 : group.getId() %>"
									style="width: 12em;"
									title="Add&nbsp;groups&hellip;"  
									tooltip="Add child groups to this group"
									disabled="<%=isEveryone || !writeMembers %>" 
								/>
								<base:button 
									id="btnRemoveMembers"
									subclass="leftaligned auto-init"
									data-auto-init="remove-link"
									data-list-id="members"
									style="width: 12em;"
									title="Remove" 
									tooltip="Remove the selected items from this group"
									disabled="<%=isEveryone || !writeMembers%>" 
								/>
							</base:buttongroup>
						</td>
					</tr>
					</table>
				</div>
			</td>
			</tr>
			</table>
		</t:tab>
		</t:tabcontrol>
		</form>
		
		<div class="legend">
			<base:icon image="required.png" />= required information
		</div>

		<base:buttongroup subclass="dialogbuttons">
			<base:button id="btnSave" title="Save" />
			<base:button id="close" title="Cancel" />
		</base:buttongroup>
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>
