/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var MergePlates = function()
{
	var merge = {};
	
	/**
		Initialize the page.
	*/
	merge.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'step-1')
		{
			Buttons.addClickHandler('btnNext', merge.gotoStep2);
			Buttons.addClickHandler('close', App.closeWindow);

			// Plate mapping
			Buttons.addClickHandler('platemapping_id.select', merge.selectPlateMapping);
			Events.addEventHandler('platemapping_id', 'base-selected', merge.setPlateMappingCallback);

			// Plate type
			Buttons.addClickHandler('platetype_id.select', merge.selectPlateType);
			Events.addEventHandler('platetype_id', 'base-selected', merge.setPlateTypeCallback);
			
		}
		else if (pageId == 'step-2')
		{
			Buttons.addClickHandler('btnMerge', merge.mergePlates);
			Buttons.addClickHandler('close', App.closeWindow);
			
			Buttons.addClickHandler('moveUp', merge.movePlateUpOrDown);
			Buttons.addClickHandler('moveDown', merge.movePlateUpOrDown);
			
		}
	}
	
	merge.validateStep1 = function()
	{
		var frm = document.forms['merge'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		if (!frm.platemapping_id.value)
		{
			Forms.showNotification('platemapping_id.select', 'You must select a plate mapping');
			return false;
		}
		if (!frm.platetype_id.value)
		{
			Forms.showNotification('platetype_id.select', 'You must select a plate type');
			return false;
		}
		return true;
	}

	merge.gotoStep2 = function()
	{
		var frm = document.forms['merge'];
		if (merge.validateStep1())
		{
			frm.submit();
		}
	}

	merge.selectPlateMapping = function()
	{
		Dialogs.selectItem('PLATEMAPPING', 'platemapping_id');
	}
	
	merge.setPlateMappingCallback = function(event)
	{
		var frm = document.forms['merge'];
		var list = frm.platemapping_id;
		if (list.length < 1 || list[0].value == '0')
		{
			Forms.addListOption(list, 0, new Option());
		}
		list[0].value = event.detail.id;
		list[0].text = event.detail.name;
		list.selectedIndex = 0;
	}
	
	merge.selectPlateType = function()
	{
		Dialogs.selectItem('PLATETYPE', 'platetype_id');
	}
	
	merge.setPlateTypeCallback = function(event)
	{
		var frm = document.forms['merge'];
		var list = frm.platetype_id;
		if (list.length < 1 || list[0].value == '0')
		{
			Forms.addListOption(list, 0, new Option());
		}
		list[0].value = event.detail.id;
		list[0].text = event.detail.name;
		list.selectedIndex = 0;
	}
	
	
	merge.validateStep2 = function()
	{
		var frm = document.forms['merge'];
		var sourceCount = Data.int('page-data', 'source-count');
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		if (frm.plates.length != sourceCount)
		{
			Forms.showNotification(frm.plates, 'You must select exactly '+sourceCount+' plates');
			return false;
		}
		return true;
	}

	merge.mergePlates = function()
	{
		var frm = document.forms['merge'];
		if (merge.validateStep2())
		{
			Forms.addHidden(frm, '+PLATE', Link.getIdsInList('plates', 'PLATE').join(','));
			frm.submit();
		}
	}

	merge.movePlateUpOrDown = function(event)
	{
		var moveDown = Data.int(event.currentTarget, 'down');
		Forms.moveListOptions('plates', moveDown);
	}

	
	return merge;
}();

Doc.onLoad(MergePlates.initPage);
